# -*- coding: utf-8 -*-
#
# # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # #
#  Noctua vagus - An adventure game with an owl as a main character         # .
#  Copyright (C) 2010  Lukáš Machyán (Phobulos@gmail.com)                   # .
#                                  (http://phobulos.wz.cz)                  # .
#                                                                           # .
#  This program is free software: you can redistribute it and/or modify     # .
#  it under the terms of the GNU General Public License as published by     # .
#  the Free Software Foundation, either version 3 of the License, or        # .
#  (at your option) any later version.                                      # .
#                                                                           # .
#  This program is distributed in the hope that it will be useful,          # .
#  but WITHOUT ANY WARRANTY; without even the implied warranty of           # .
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the            # .
#  GNU General Public License for more details.                             # .
#                                                                           # .
#  You should have received a copy of the GNU General Public License        # .
#  along with this program. If not, see <http://www.gnu.org/licenses/>.     # .
#                                                                           # .
# # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # .
# . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . .

import os, pygame, random, math, datetime
try:
    import pygame.gfxdraw
except:
    pass
    
from pygame.locals import *

import main, menu

def nahrat_obrazek(jmeno, subdir = 'images', klicovaBarva=None):
    plneJmeno = os.path.join('data', subdir, jmeno)  # připojení umístění ke jménu obrázku
    try:
        obrazek = pygame.image.load(plneJmeno)
    except:
        main.screen_menu = menu.Error_screen(s = plneJmeno, nr = 1)
        return None
    else:
        obrazek = obrazek.convert_alpha()
        if not klicovaBarva:
            klicovaBarva = obrazek.get_at((0,0))
        obrazek.set_colorkey(klicovaBarva, RLEACCEL)
        return obrazek

# Nahrávání více obrázků pro animaci
def nahrat_obrazky(imgName):
    obrazky = []
    c = 0
    if os.path.isfile(os.path.join('data', 'images', imgName)):
        c += 1
    try:
        if os.path.isfile(os.path.join('data', 'images', imgName % 0)):
            c += 2
    except:
        pass
    
    if c == 0:  # No image found
        try:  # Try to replace %d with 0
            p = os.path.join('data', 'images', imgName % 0)
        except:
            p = os.path.join('data', 'images', imgName)
        main.screen_menu = menu.Error_screen(s = p, nr = 1)
        return None   
    elif c == 1:  # Image without number found
        obrazky.append(nahrat_obrazek(imgName))
    else:  # Image without and/or with number found
        i = 0
        while os.path.isfile(os.path.join('data', 'images', imgName % i)):
            fullImgName = os.path.join('data', 'images', imgName % i)
            tmpImage = pygame.image.load(fullImgName)
            tmpImage = tmpImage.convert_alpha()
            obrazky.append(tmpImage)
            i += 1
        
##    if os.path.isfile(os.path.join('data', 'images', imgName)):
#    try:
#        fullImgName = os.path.join('data', 'images', imgName)
#        tmpImage = pygame.image.load(fullImgName)
#        tmpImage = tmpImage.convert_alpha()
#        obrazky.append(tmpImage)
##        obrazky.append(nahrat_obrazek(imgName))
##    else:
#    except:
#        try:
#            i = 0
#            while os.path.isfile(os.path.join('data', 'images', imgName % i)):
#                fullImgName = os.path.join('data', 'images', imgName % i)
#                tmpImage = pygame.image.load(fullImgName)
#                tmpImage = tmpImage.convert_alpha()
#                obrazky.append(tmpImage)
#                i += 1
#        except:
#            main.screen_menu = menu.Error_screen(nr = 1)
#            return None
#    for i in range(pocet):
#        fullImgName = os.path.join('data', 'images', imgName % i)
#        tmpImage = pygame.image.load(fullImgName)
#        tmpImage = tmpImage.convert_alpha()
#        obrazky.append(tmpImage)
    return obrazky

# Nahrávání zvuků	
def nahrat_zvuk(jmeno):
    class NoSound:
        def play(self, r = 0): pass
        def set_volume(self, v = 1): pass
        def stop(self): pass
    if not pygame.mixer or pygame.mixer.get_init() == None:
        return NoSound()
    plneJmeno = os.path.join('data', 'sounds', jmeno)
    try:
        zvuk = pygame.mixer.Sound(plneJmeno)  # When this attempt is unsuccesful the game could work without sounds
    except:
        print (_('Sound could not be loaded: {0}').decode(main.CODING).format(plneJmeno))
        return NoSound()
    return zvuk

# Export functions
def make_screenshot():
    pygame.image.save(main.obrazovka, os.path.join(main.save_dir, "screenshots", str(datetime.date.today()) + "_" + str(datetime.datetime.now().hour) + "-" + str(datetime.datetime.now().minute) + "-" + str(datetime.datetime.now().second) + "-" + str(datetime.datetime.now().microsecond) + ".png"))
    # Později ochrana proti chybám, volba grafického formátu apod.


# Drawing functions
def draw_circle(surf, col, pos, radius, width):

    """Try to draw antialiased circle from experimental module (pygame 1.9.0) gfxdraw. \n
    If an error occurs draw normal circle."""

    try:
        pygame.gfxdraw.aacircle(surf, pos[0], pos[1], radius, col)  # Line width isn't here :-(
    except:
        pygame.draw.circle(surf, col, pos, radius, width)

def draw_emblem(surf, pos, embl, col = (255, 0, 0), line_width = 1, scale = 1):
    l = len(embl)
    
    if l > 0:
        pygame.draw.rect(surf, (0, 100, 20), (pos[0] - int(10 * scale), pos[1] - int(10 * scale), int(20 * scale), int(20 * scale)))
    
    for i in range(l):
        p = [pos[0], pos[1]]
        e = embl[i] % 10
    
        if e == 1:
            p[0] += int(10 * scale)
        elif e == 2:
            p[1] -= int(10 * scale)
        elif e == 3:
            p[0] -= int(10 * scale)
        elif e == 4:
            p[1] += int(10 * scale)
                 
        if embl[i] >= 10 and embl[i] < 20:
            draw_circle(surf, col, p, int(10 * scale), line_width)
        elif embl[i] >= 20 and embl[i] < 30:
            pygame.draw.line(surf, col, (p[0] - int(10 * scale), p[1]), (p[0] + int(10 * scale), p[1]), line_width)
        elif embl[i] >= 30 and embl[i] < 40:
            pygame.draw.line(surf, col, (p[0], p[1] - int(10 * scale)), (p[0], p[1] + int(10 * scale)), line_width)
            
def draw_HUD(surf, pos):
    main.text = main.pismo.render("Variable: " + str(main.timer[0]), 1, main.font_color)
    surf.blit(main.text, (pos[0] + main.SCREEN_WIDTH/2, pos[1]))
    
    main.text = main.pismo_small.render(_("fps: ").decode(main.CODING) + str(round(main.fps2, 2)), 1, main.font_color)
    text_pos = main.text.get_rect()
    text_pos.top = 10
    text_pos.right = main.SCREEN_WIDTH - 10
    surf.blit(main.text, text_pos)
    
    if main.info_message != "":
        main.text = main.pismo.render(main.info_message, 1, main.font_color)
        text_pos = main.text.get_rect()
        text_pos.right = main.SCREEN_WIDTH - 20
        text_pos.top = 50
        surf.blit(main.text, text_pos)
        
    if main.quest_mark_pos != None:     # Drawing circle around objective
        if main.quest_mark_nr < 600:
            main.quest_mark_nr += 8
            pygame.draw.circle(surf, (200, 200, 100), main.quest_mark_pos, main.quest_mark_nr, 2)
        else:
            main.quest_mark_pos = None
            main.quest_mark_nr = 0

#    main.text = main.pismo.render(_("group[active_room]: ") + str(main.group[main.active_room].sprites), 1, (10, 60, 10))
#    surf.blit(main.text, (pos[0], pos[1] + 100))

def change_quest_status(q_name, q_status, q_pos = None):
    if q_status == -1:
        if q_name not in main.quest:  # This objective isn't exists
            main.quest[q_name] = q_status  # Create it
            main.info_message = _("New objective discovered").decode(main.CODING)  # Show message
            main.sova.info_message_time = 400
            main.quest_mark_pos = q_pos
            
    elif q_status == 1:
        if main.quest[q_name] != q_status:  # This objective doesn't have needed value
            main.quest[q_name] = q_status  # Change it
            main.info_message = _("Objective completed").decode(main.CODING)  # Message
            main.sova.info_message_time = 400
            
    else:
        if main.quest[q_name] != q_status:  # This objective doesn't have needed value
            main.quest[q_name] = q_status  # Change it

# Generování stromů v pozadí
def generovat_strom(posun_x = 200, min_sirka = 30, max_sirka = 40, min_krok = 5, max_krok = 10, min_stoup = 1, max_stoup = 5):
    random.seed()
    kroky = int(math.ceil(min_krok + (max_krok - min_krok) * random.random()))
    stoup = min_stoup + (max_stoup - min_stoup) * random.random()
    
    body = []
    for i in range(kroky + 1):
        x = - (min_sirka/2 + (max_sirka/2 - min_sirka/2) * random.random()) - i * stoup + posun_x
        y = (480 / kroky) * i
        body.append((x, y))
                # 480 - výška obrazu (může být větší, pokud bude dole vznikat mezera při vyšším počtu kroků
    for i in range(kroky + 1):
        x = (min_sirka/2 + (max_sirka/2 - min_sirka/2) * random.random()) + (kroky - i) * stoup + posun_x
        y = (480 / kroky) * (kroky - i)
        body.append((x, y))

    return body

def generovat_strom2(posun_x = 200, min_sirka = 50, max_sirka = 50, min_alfa = -15, max_alfa = 15, min_krok = 50, max_krok = 50, max_beta = 7):
    sirka = min_sirka + (max_sirka - min_sirka) * random.random()
    
    x = -sirka/2
    y = 0
    x2 = abs(x)

    body = []
    body.append((posun_x + x, y))

    body2 = []
    body2.append((posun_x + x2, y))

    while y <= 480:
        alfa = min_alfa + (max_alfa - min_alfa) * random.random()
        beta = alfa + max_beta * random.random()
        
        x1 = (min_krok + (max_krok - min_krok) * random.random()) * math.sin(math.radians(alfa))
        y1 = (min_krok + (max_krok - min_krok) * random.random()) * math.cos(math.radians(alfa))

        x21 = (min_krok + (max_krok - min_krok) * random.random()) * math.sin(math.radians(beta))

        x = x + x1
        y = y + y1
        x2 = x2 + x21

        body.append((posun_x + x, y))
        body2.append((posun_x + x2, y))
    
    body2.reverse()
    body = body + body2

    return body
        
