# -*- coding: utf-8 -*-
#
# # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # #
#  Noctua vagus - An adventure game with an owl as a main character         # .
#  Copyright (C) 2010  Lukáš Machyán (Phobulos@gmail.com)                   # .
#                                  (http://phobulos.wz.cz)                  # .
#                                                                           # .
#  This program is free software: you can redistribute it and/or modify     # .
#  it under the terms of the GNU General Public License as published by     # .
#  the Free Software Foundation, either version 3 of the License, or        # .
#  (at your option) any later version.                                      # .
#                                                                           # .
#  This program is distributed in the hope that it will be useful,          # .
#  but WITHOUT ANY WARRANTY; without even the implied warranty of           # .
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the            # .
#  GNU General Public License for more details.                             # .
#                                                                           # .
#  You should have received a copy of the GNU General Public License        # .
#  along with this program. If not, see <http://www.gnu.org/licenses/>.     # .
#                                                                           # .
# # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # .
# . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . .

import os, pygame, gettext

from pygame.locals import *

import files, main, data, menu

from objects import *
from rooms import *

def init():

# Variables
    # Constants
    main.NAME = "Noctua vagus"
    main.NAME2 = "noctua-vagus"
    main.AUTHOR = "Phobulos"
    main.VERSION = "0.1.2"
    main.YEAR = "2010"
    main.CODING = "utf-8"
    main.SCREEN_WIDTH = 854
    main.SCREEN_HEIGHT = 480
    
    # Option variables - changeable in config file
    main.term = 0        # showing messages on terminal
    main.long_click_delay = 8
    main.ovladaci_klavesy = [0, K_RIGHT, K_UP, K_LEFT, K_DOWN, K_LSHIFT, K_x, K_c, K_v, K_SPACE, K_TAB, K_F11]
                         #   0     1      2       3      4         5      6    7    8      9       10    11
    main.language = "auto"
    main.font_color = (170, 170, 140)   # pygame.Color(170, 170, 140) - umožní lepší manipulaci s barvou
    main.font_color_active = (255, 160, 0)
    main.font_color_link = (220, 220, 150)
    main.back_color = (20, 20, 20, 150)
    main.back_color_question = (40, 40, 20, 150)
    main.back_color_objectives = (30, 150, 35, 150)
    main.back_color_error = (100, 0, 0, 255)
    
    main.max_fps = 40
    main.font_name = "texgyreadventor-bold.otf"
    main.locale_dir = 'locale'  # This could be replaced with /usr/share/locale for example
    main.line_spacing = 25  #px
    main.line_spacing_small = 15  #px
    
    # Variables - changing during game
    main.klavesa = [0, 0, 0, 0, 0, 0]
    main.filtr_barvy = (0, 0, 0, 0)

    main.group = []
    main.owl_group = pygame.sprite.OrderedUpdates()
    
    main.quest = {}
    
    main.info_message = ""
    main.quest_mark_pos = None
    main.quest_mark_nr = 0
    
    # Directory for saving data, work with configuration file
        # strings can't be localized with gettext because used language is read 
        # from config file on this lines
    try:
        # try to set saving directory with enviroment variable "home" (Linux. I hope every Unix too.)
        main.save_dir = os.path.join(os.environ['HOME'], ".phobulos", main.NAME2)
    except:
        # enviroment variable "home" doesn't exists
        try:
            # try to set saving directory with enviroment variable "appdata" (Windows)
            main.save_dir = os.path.join(os.environ['APPDATA'], ".phobulos", main.NAME2)
        except:
            # enviroment variable "appdata" doesn't exists - saving dir in game directory (other OS)
            main.save_dir = "config"

    # Making saving directory
    if not os.access(main.save_dir, os.F_OK):   # Make saving directory if it doesn't exists
        os.makedirs(main.save_dir)              # Maybe this part should be rewritten
        if os.access(main.save_dir, os.F_OK):   # but it suffice for now
            try:
                if main.term:
                    print (("Directory {0} for data saving created").format(main.save_dir))
            except:
                pass
    
    # Work with configuration file
    if not os.access(os.path.join(main.save_dir, "config.cfg"), os.F_OK):  # config file doesn't exists
        files.write_config(os.path.join(main.save_dir, "config.cfg"))      # write config to file (will also take place after config change in game)
        try:
            if main.term:
                print (("Configuration file created in {0}").format(main.save_dir))  # message
        except:
            pass
    else:                                                                   # config file exists
        try:
            files.read_config(os.path.join(main.save_dir, "config.cfg"))   # try to read config...
        except:                                                            # if failed (old version of file, corrupted file)...
            files.write_config(os.path.join(main.save_dir, "config.cfg"))  # ... write config again (!every option should be checked individually!)
            
        try:
            if main.term:
                print (("Configuration has been read from {0}").format(os.path.join(main.save_dir, "config.cfg")))  # message
        except:
            pass
    # Create screenshot directory
    if not os.access(os.path.join(main.save_dir, "screenshots"), os.F_OK):
        os.mkdir(os.path.join(main.save_dir, "screenshots"))
        
    # Localisation
    if main.language == "auto":
        t = gettext.translation(main.NAME2, main.locale_dir, fallback = 1)
    else:
        t = gettext.translation(main.NAME2, main.locale_dir, languages = [main.language], fallback = 1)
    _ = t.ugettext
    t.install()
    main.translators = _("translator-credits")
    # Because of previous localisation statement in this file, decoding of strings 
    # (string.decode("utf-8") in this file isn't needed.
    
    # Variables set according other variables
    main.fps = main.max_fps
    main.font_path = os.path.join("data", "fonts", main.font_name)
    
    if main.term:  # Unimportant info message about game start
        print (_("Game started"))
            
    # Quest strings
    main.quest_texts = {
    'little_bird0': _("Little bird has fallen out its nest."), 
    'little_bird1': _("The owl has taken it back to the nest.")
    }
    
    # Graphics
    pygame.init()
    os.environ["SDL_VIDEO_CENTERED"] = "1"  # Center window
    main.obrazovka = pygame.display.set_mode((main.SCREEN_WIDTH, main.SCREEN_HEIGHT))
    pygame.display.set_caption(("{0} {1}").format(main.NAME, main.VERSION))
    pygame.display.set_icon(data.nahrat_obrazek('16x16.png', subdir = 'icons'))
    
    # Objects
    main.sova = Sova()
    main.obj = []
    main.creature = []

    # Rooms
    main.room = []
    main.room.append(Doma())            #  r   u   l   d
    main.room[len(main.room) - 1].exits = (5, 1, -1, 4) # 0
    main.room[len(main.room) - 1].activated()  # Needed for building enviroment
    
    main.room.append(StredLesa())
    main.room[len(main.room) - 1].exits = (-1, 2, -1, 0) # 1
    main.room.append(NadStromy())
    main.room[len(main.room) - 1].exits = (-1, 3, -1, 1) # 2
    main.room.append(Nebe())
    main.room[len(main.room) - 1].exits = (-1, -1, -1, 2) # 3 - nemá vliv pro průchod sovy - mohly by využít padající předměty
    main.room.append(PodStromem())
    main.room[len(main.room) - 1].exits = (-1, 0, -1, -1) # 4
    main.room.append(Jezirko())
    main.room[len(main.room) - 1].exits = (6, -1, 0, -1) # 5
    main.room.append(Potok())
    main.room[len(main.room) - 1].exits = (-1, -1, 5, -1) # 6
        
    main.active_room = 0
    
    # Sounds
    main.houkani = [None, data.nahrat_zvuk('houkani1.ogg'), data.nahrat_zvuk('houkani2.ogg'), data.nahrat_zvuk('houkani3.ogg')]
#    main.snd_wingflap = data.nahrat_zvuk('wingflap2.ogg')  # wing flaps considered but it sounds strenge
    
    # Text
#    if pygame.font: # pokusný text
    main.pismo = pygame.font.Font(main.font_path, 22)
    main.pismo_big = pygame.font.Font(main.font_path, 38)
    main.pismo_small = pygame.font.Font(main.font_path, 14)
#        main.pismo = pygame.font.Font(os.path.join('data', 'fonts', "texgyretermes-bold.otf"), 20)

    # Enter to main menu
    main.screen_menu = menu.Menu_screen()
