# -*- coding: utf-8 -*-
#
# # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # #
#  Noctua vagus - An adventure game with an owl as a main character         # .
#  Copyright (C) 2010  Lukáš Machyán (Phobulos@gmail.com)                   # .
#                                  (http://phobulos.wz.cz)                  # .
#                                                                           # .
#  This program is free software: you can redistribute it and/or modify     # .
#  it under the terms of the GNU General Public License as published by     # .
#  the Free Software Foundation, either version 3 of the License, or        # .
#  (at your option) any later version.                                      # .
#                                                                           # .
#  This program is distributed in the hope that it will be useful,          # .
#  but WITHOUT ANY WARRANTY; without even the implied warranty of           # .
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the            # .
#  GNU General Public License for more details.                             # .
#                                                                           # .
#  You should have received a copy of the GNU General Public License        # .
#  along with this program. If not, see <http://www.gnu.org/licenses/>.     # .
#                                                                           # .
# # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # .
# . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . .

import pygame, os, random
from pygame.locals import *

import data, main
from main import *

from objects import *
from rooms import *

class Room:
    
    def __init__(self):
        self.ide = len(main.room)
        self.block = []   # obstacle for falling subjects
        self.block2 = []  # obstacle for owl
        main.group.append(pygame.sprite.LayeredUpdates(()))
        self.build()
#        self.build_env() - now called after activation
        
    def activated(self):
        """This room has been activated - we have come to it."""
        self.build_env()  # rebuilt background and foreground
        
    def build(self):
        pass
        
    def build_env(self):
        pass
        
    def deactivated(self):
        """This occurs when this room is left."""
        pass
        
    def leave(self, smer):
        if self.exits[smer - 1] != -1:      # It is possible to leave in this direction
            del self.pozadi  # delete this room background
            del self.popredi  # delete this room foreground
            main.active_room = self.exits[smer - 1]     # Change room
            main.room[main.active_room].activated()
            main.sova.wrap(smer)                        # Move the owl to opposite side
            self.deactivated()
            
            # Maybe some subjects fell from room above this room but they didn't check "objective complete" collision
            # This is checked here when owl comes to room.
            for i in range(len(main.group[main.active_room].sprites())):
                try:
                    main.group[main.active_room].sprites()[i].special_collision()
                except:
                    pass

            if main.quest_mark_pos != None:     # Hide mark of new objective 
                main.quest_mark_pos = None      # after leaving room
                
        else:                              # It is NOT possible to leave in this direction
            main.sova.stop(smer)           # stop owl motion
            
    def new_subject(self, subj, l = 0): # l = layer; bigger l => nearer to player
        main.obj.append(subj)
#        main.obj[len(main.obj) - 1].place = self.ide
#        main.obj[len(main.obj) - 1].layer = l
        main.group[len(main.group) - 1].add(main.obj[len(main.obj) - 1], layer = l)
        return main.obj[-1]   # allows to change some object variables after creation
        
    def new_creature(self, subj, l = -10):
        main.creature.append(subj)
#        main.creature[len(main.creature) - 1].place = self.ide
#        main.obj[len(main.obj) - 1].layer = l
        main.group[len(main.group) - 1].add(main.creature[len(main.creature) - 1], layer = l)
        return main.creature[-1]
        
#def change_room(r):
#    #if main.sova.inventory != 0:
#    #    main.sova.inventory.presun(r)
#    main.active_room = r
#    main.sova.wrap()

class Doma(Room):
    
    def build(self):
    
        """Build environment of this room with background, foreground and all objects."""
    
#        for i in range(5):
#            # Subjects
#            self.new_subject(Orech((200 + 200 * random.random(), 200), embl = (10, 20, 32)))
        
#        # Creatures
#        main.creature.append(Veverka())
#        main.creature[len(main.creature) - 1].place = self.ide

        # Bloky
        self.block.append((400, 220, 70, 20))
#        pygame.draw.rect(self.pozadi, (255, 0, 0), self.block[0], 1) - If you want to see this block
#        self.block2.append((400, 220, 70, 20)) - Nyní nepoužitý blok pro blokaci sovy
        
    def build_env(self):
        # Pozadí
        self.pozadi = (pygame.Surface(main.obrazovka.get_size())).convert()
        self.pozadi.fill((30, 30, 35)) # ((50, 50, 90)) - modř
    
        # Stromy v pozadí
        pygame.draw.polygon(self.pozadi, (30, 10, 10), (data.generovat_strom2(10, 10, 15, -7, 7, 10, 20, 3)))
        pygame.draw.polygon(self.pozadi, (30, 10, 10), (data.generovat_strom2(250, 15, 20, -8, 8, 7, 15, 3)))
        pygame.draw.polygon(self.pozadi, (30, 10, 10), (data.generovat_strom2(750, 15, 20, -8, 8, 7, 15, 3)))
        pygame.draw.polygon(self.pozadi, (30, 10, 10), (data.generovat_strom2(650, 15, 20, -8, 8, 7, 15, 3)))
        
        pygame.draw.polygon(self.pozadi, (50, 20, 10), (data.generovat_strom2(100, 30, 35, -7, 7, 10, 20, 3)))
        pygame.draw.polygon(self.pozadi, (50, 20, 10), (data.generovat_strom2(300, 40, 50, -12, 12, 25, 60, 10)))
        pygame.draw.polygon(self.pozadi, (50, 20, 10), (data.generovat_strom2(600, 40, 50, -12, 12, 40, 60, 4)))
        pygame.draw.polygon(self.pozadi, (50, 20, 10), (data.generovat_strom2(800, 50, 60, -15, 15, 25, 60, 10)))
    
        # stromy
        obr_strom = data.nahrat_obrazek("strom.png")
        if obr_strom != None:
            poz_strom = obr_strom.get_rect()
            poz_strom.centerx = self.pozadi.get_rect().centerx  # vycentrování stromu
            self.pozadi.blit(obr_strom, poz_strom)

        # Popředí
        self.popredi = (pygame.Surface(main.obrazovka.get_size())).convert_alpha()
        self.popredi.fill((0, 0, 30, 100))
            
class StredLesa(Room):
    def build(self):
        
        # Objects
        self.new_subject(Hnizdo((450, 180), 0), 10)
        self.new_subject(Ptace((418, 160), 0))
        self.new_subject(Ptace((478, 160), 0))
        
        # Blocks
        self.block.append((440, 180, 20, 20))
#        pygame.draw.rect(self.pozadi, (255, 0, 0), self.block[0], 1)

    def build_env(self):
        # Pozadí
        self.pozadi = (pygame.Surface(main.obrazovka.get_size())).convert()
        self.pozadi.fill((40, 40, 50)) # ((50, 50, 90)) - modř
    
        # Stromy v pozadí
        pygame.draw.polygon(self.pozadi, (30, 10, 10), (data.generovat_strom2(10, 10, 15, -7, 7, 10, 20, 3)))
        pygame.draw.polygon(self.pozadi, (30, 10, 10), (data.generovat_strom2(250, 15, 20, -8, 8, 7, 15, 3)))
        pygame.draw.polygon(self.pozadi, (30, 10, 10), (data.generovat_strom2(750, 15, 20, -8, 8, 7, 15, 3)))
        pygame.draw.polygon(self.pozadi, (30, 10, 10), (data.generovat_strom2(650, 15, 20, -8, 8, 7, 15, 3)))
        
        pygame.draw.polygon(self.pozadi, (50, 20, 10), (data.generovat_strom2(100, 30, 35, -7, 7, 10, 20, 3)))
        pygame.draw.polygon(self.pozadi, (50, 20, 10), (data.generovat_strom2(300, 40, 50, -12, 12, 25, 60, 10)))
        pygame.draw.polygon(self.pozadi, (50, 20, 10), (data.generovat_strom2(600, 40, 50, -12, 12, 40, 60, 4)))
        pygame.draw.polygon(self.pozadi, (50, 20, 10), (data.generovat_strom2(800, 50, 60, -15, 15, 25, 60, 10)))
    
        # stromy
        obr_strom = data.nahrat_obrazek("strom_stred.png")
        if obr_strom != None:
            poz_strom = obr_strom.get_rect()
            poz_strom.centerx = self.pozadi.get_rect().centerx  # vycentrování stromu
            self.pozadi.blit(obr_strom, poz_strom)
        # back part of nest
        obr = data.nahrat_obrazek("hnizdo_back.png")
        if obr != None:
            poz = obr.get_rect()
            poz.center = (450, 165)
            self.pozadi.blit(obr, poz)
    
        # Popředí
        self.popredi = (pygame.Surface(main.obrazovka.get_size())).convert_alpha()
        self.popredi.fill((0, 0, 30, 80))
            
class NadStromy(Room):
    def build(self):
        pass
                
    def build_env(self):
        # Pozadí
        self.pozadi = (pygame.Surface(main.obrazovka.get_size())).convert_alpha()
        self.pozadi.fill((10, 10, 50)) # ((50, 50, 90)) - modř
        
        for i in range(100):
            random.seed()
            x = int(random.random() * main.SCREEN_WIDTH)
            y = int(random.random() * main.SCREEN_HEIGHT)
            self.pozadi.set_at((x, y), (255, 255, 0))        
        
        obr = data.nahrat_obrazek("mesic.png")
        if obr != None:
            poz = obr.get_rect()
            poz.left = 500
            poz.top = 80
            self.pozadi.blit(obr, poz)
        
        obr = data.nahrat_obrazek("strom_vrsek1.png")
        if obr != None:
            poz = obr.get_rect()
            poz.left = -40
            poz.bottom = main.SCREEN_HEIGHT
            self.pozadi.blit(obr, poz)
        
        obr = data.nahrat_obrazek("strom_vrsek2.png")
        if obr != None:
            poz = obr.get_rect()
            poz.left = 100
            poz.bottom = main.SCREEN_HEIGHT
            self.pozadi.blit(obr, poz)
        
        obr = data.nahrat_obrazek("strom_vrsek4.png")
        if obr != None:
            poz = obr.get_rect()
            poz.left = 210
            poz.bottom = main.SCREEN_HEIGHT
            self.pozadi.blit(obr, poz)
        
        obr = data.nahrat_obrazek("strom_vrsek.png")
        if obr != None:
            poz = obr.get_rect()
            poz.centerx = main.SCREEN_WIDTH/2
            poz.bottom = main.SCREEN_HEIGHT
            self.pozadi.blit(obr, poz)
        
        obr = data.nahrat_obrazek("strom_vrsek3.png")
        if obr != None:
            poz = obr.get_rect()
            poz.left = 520
            poz.bottom = main.SCREEN_HEIGHT
            self.pozadi.blit(obr, poz)
        
        obr = data.nahrat_obrazek("strom_vrsek4.png")
        if obr != None:
            poz = obr.get_rect()
            poz.left = 650
            poz.bottom = main.SCREEN_HEIGHT
            self.pozadi.blit(obr, poz)
        
        obr = data.nahrat_obrazek("strom_vrsek2.png")
        if obr != None:
            poz = obr.get_rect()
            poz.left = 780
            poz.bottom = main.SCREEN_HEIGHT
            self.pozadi.blit(obr, poz)
        
        # Popředí
        self.popredi = (pygame.Surface(main.obrazovka.get_size())).convert_alpha()
        self.popredi.fill((50, 50, 0, 80))
            
class Nebe(Room):
    def build(self):
        self.over_trees = 0
        
    def build_env(self):
        self.pozadi = (pygame.Surface(main.obrazovka.get_size())).convert_alpha()
        self.star_field()
        
    def star_field(self):
        # Pozadí
        self.pozadi.fill((10, 10, 50)) # ((50, 50, 90)) - modř
        
        for i in range(100):
            random.seed()
            x = int(random.random() * main.SCREEN_WIDTH)
            y = int(random.random() * main.SCREEN_HEIGHT)
            self.pozadi.set_at((x, y), (255, 255, 0))
            
        # Popředí
        self.popredi = (pygame.Surface(main.obrazovka.get_size())).convert_alpha()
        self.popredi.fill((50, 50, 0, 80))
        
    def leave(self, smer):
        if smer == 2:
            main.sova.wrap(smer)
            self.over_trees += 1
        elif smer == 4:
            if self.over_trees > 0:
                main.sova.wrap(smer)
                self.over_trees -= 1
            else:
                del self.pozadi  # delete this room background
                del self.popredi  # delete this room foreground
                main.active_room = 2
                main.room[main.active_room].activated()
                main.sova.wrap(smer)
        
        else:
            main.sova.stop(smer)
            
class PodStromem(Room):
    def build(self):
        # Creatures
        self.new_creature(Veverka((750, 400)))
#        main.creature.append(Veverka())
#        main.creature[len(main.creature) - 1].place = self.ide
#        main.creature[len(main.creature) - 1].rect.bottomright = (750, 400)
        
        # Subjects
        self.new_subject(Ptace((450, 450)))
        
        # Bloky
        self.block.append((80, 220, 50, 15))
#        pygame.draw.rect(self.pozadi, (255, 0, 0), self.block[0], 1)  # If you want to see this block
        #self.block.append((0, 300, main.SCREEN_WIDTH, main.SCREEN_HEIGHT - 300))
        
    def build_env(self):
        # Pozadí
        self.pozadi = (pygame.Surface(main.obrazovka.get_size())).convert()
        obr = data.nahrat_obrazek("back_podstromem.png")
        if obr != None:
            poz = obr.get_rect()
            self.pozadi.blit(obr, poz)
        
        # Popředí
        self.popredi = (pygame.Surface(main.obrazovka.get_size())).convert_alpha()
        self.popredi.fill((0, 0, 30, 100))
        
class Jezirko(Room):
    
    def activated(self):
        Room.activated(self)
        self.snd_water = data.nahrat_zvuk("water.ogg")
        self.snd_water.play(-1)
        self.snd_water.set_volume(0.2)
    
    def build(self):
#        self.new_subject(WaterPool((427, 258), ani_rew = 0))  # hidden at start - will appear later
        pass
        
    def build_env(self):
        # Pozadí
        self.pozadi = (pygame.Surface(main.obrazovka.get_size())).convert()
        obr = data.nahrat_obrazek("back_pool.jpg")
        if obr != None:
            poz = obr.get_rect()
            self.pozadi.blit(obr, poz)
        
        # Popředí
        self.popredi = (pygame.Surface(main.obrazovka.get_size())).convert_alpha()
        self.popredi.fill((0, 0, 30, 100))
        
    def deactivated(self):
        self.snd_water.stop()
        del self.snd_water
        Room.deactivated(self)
        
class Potok(Room):

    def activated(self):
        Room.activated(self)
        self.snd_water = data.nahrat_zvuk("water.ogg")
        self.snd_water.play(-1)

    def build(self):
        # Subjects
        self.new_subject(Stream((490, 131), ani_rew = 0))
        self.new_subject(Waterfall((564, 329), ani_rew = 0))
#        self.new_subject(Waterfall2((255, 329), ani_rew = 0))
        self.new_subject(Kamen((334, 234)))
        
    # Blocks
        self.block.append((789, 229, 64, 50))
        self.block.append((329, 309, 100, 16))

    def deactivated(self):
        self.snd_water.stop()
        del self.snd_water
        Room.deactivated(self)

    def build_env(self):
        # Pozadí
        self.pozadi = (pygame.Surface(main.obrazovka.get_size())).convert()
        obr = data.nahrat_obrazek("back_potok.jpg")
        if obr != None:
            poz = obr.get_rect()
            self.pozadi.blit(obr, poz)
        
        # Popředí
        self.popredi = (pygame.Surface(main.obrazovka.get_size())).convert_alpha()
        self.popredi.fill((0, 0, 30, 100))
